function [sys,x0] = CSTR_sedo(t,x,u,flag,Estado_Estacionario)

% Planta: CSTR
%
% Autor:  Ricardo Kalid
%         kalid@ufba.br
%
% Interface no Simulink para  CSTR_sedo.m     
%
% Inputs:
%
% t : tempo em segundos
% x : estados do sistema
% u : perturbaes externas ou variveis manipuladas
%
% Outputs:   sys  e  x0  como descritos no manual do SIMULINK
%            quando  flag    0 , sys contem as dimenses dos vetores e 
%            x0  contem as condies iniciais,
%            quando  flag    1 , sys contem as derivadas,
%            quando  flag    3 , sys contem as sadas: 

% Autor:  Ricardo Kalid
%         kalid@ufba.br

A = 1 ; % reagente
B = 2 ; % produto
C = 3 ; % inerte

% Nmero de Variveis Manipuladas
% Autor:  Ricardo Kalid
%         kalid@ufba.br

% Dados para a   PLANTA   do CSTR
% Exemplo modificado a partir de exemplo 9-4 do Corripio pg 471 a 474
Vi    =    2.0     ; % [=] m
ro    =   19.20    ; % [=] kmol/m
rom   =    1.00    ; % [=] ton/m - mistura reacinal: A + B + C
Cp    =    1.815e5 ; % [=] J/(kmol.C)
Area  =    5.40    ; % [=] m - area de troca termica - area da serpentina
roc   = 1000.00    ; % [=] kg/m  - fluido termico
dHR   =  2*9.86e7  ; % [=] J/kmol         { original -9.86e7 }
ko    =  267.84/10 ; % [=] m/(h.kmol)    { original 0.0744 m/(s.kmol) = 267.84 m/(h.kmol) }
Ea    =    1.182e7 ; % [=] J/kmol         ( original 1.182e7 J/kmol }
U     =    1.278e7 ; % [=] J/(h.m.C)    { original 3550 J/(s.m.oC) = 1.278e7 J/(h.m.C) }
Vc    =   18.0     ; % [=] m
Cpc   = 4184.00    ; % [=] K/(kg.C)

Rg    = 8314.39    ; % [=] 
g     =    9.81    ; % [=] m/s

Rt    = ( 2 * Vi / ( 2 * pi ) ) ^ (1/3) ; % Lmax = 2R , e L = 50% Lmax
At    = pi * Rt ^ 2                     ; % rea transversal do tanque (L=2R) [=] m
Lt    = Vi / At                         ; % nvel de projeto [=] m

if abs(flag) == 1
% Clculo das derivadas dos estados
x = x .* [ x > 0 ] ;
u = u .* [ u > 0 ] ;

% Estados do sistema
L     = x(1) ; % nvel        do reator     [=] m
CM(A) = x(2) ; % concentrao do reagente A [=] kmol/m
CM(B) = x(3) ; % concentrao do produto  B [=] kmol/m
CM(C) = x(4) ; % concentrao do inerte   C [=] kmol/m
T     = x(5) ; % temperatura  do reator     [=] C
Tc    = x(6) ; % temperatura  da camisa     [=] C

% Entradas do sistema
F     = u(1) ; % vazo        da descarga fornecida    [=] ton/h
Fcat  = u(2) ; % vazao        do catalisador           [=] kg/h   -  valor inicial recomendado  1,0 kg/h
Fi    = u(3) ; % vazo        da carga                 [=] ton/h
Ti    = u(4) ; % temperatura  da carga (make-up)       [=] C
xi(A) = u(5) ; % fracao molar do reagente A (make-up)  [=] (mol de A) / (mol mistura)
xi(B) = u(6) ; % fracao molar do produto  B (make-up)  [=] (mol de B) / (mol mistura)
xi(C) = u(7) ; % fracao molar do inerte   C (make-up)  [=] (mol de C) / (mol mistura)
Tci   = u(8) ; % temperatura  do fluido de aquecimento [=] C
Fc    = u(9) ; % vazo        do fluido de aquecimento [=] m/h

if ( L <= eps | CM(A) < 0.0 | CM(B) < 0.0 | CM(C) < 0.0 | T <= eps | Tc <= eps )
   disp( '      F        Fc       L        CA        CB        CC        T        Tc ' )
   disp( [ F  Fc  L  CM(A)  CM(B)  CM(C)  T  Tc ] )
   disp( ' Problemas em CSTR_sedo.M' )
   return
end

% Fracao molar na descarga
xM   = CM / sum(CM) ; % fraes molares

% Vazao molar
Fi_M = xi * Fi * ro / rom ; % [=] kmol/h
F_M  = xM * F  * ro / rom ; % [=] kmol/h

% Sistema de Equaes Diferenciais Ordinrias (SEDO)
V     = At * L                                      ;
Eat   = Ea / Fcat                                   ;
k     = ko * exp( - Eat / ( Rg * ( T + 273.15 ) ) ) ;
G     = k * CM(A)                                   ;
UA    = Fc * U * Area                               ; % U proporcinal a vazao do fluido de aquecimento

dLdt  = ( Fi      - F      ) / At                                                                          ;
dCAdt = ( ( Fi_M(A) - F_M(A) )  -  CM(A) * ( Fi - F ) /  rom ) / V   -  G                                  ;
dCBdt = ( ( Fi_M(B) - F_M(B) )  -  CM(B) * ( Fi - F ) /  rom ) / V   +  G                                  ;
dCCdt = ( ( Fi_M(C) - F_M(C) )  -  CM(C) * ( Fi - F ) /  rom ) / V                                         ;
dTdt  = Fi * ( Ti  - T  ) / ( rom * V )  -  dHR * G / ( ro * Cp )  -  UA * ( T - Tc ) / ( V * ro * Cp )    ;
dTcdt = Fc * ( Tci - Tc ) / ( roc/1000 * Vc )                      +  UA * ( T - Tc ) / ( Vc * roc * Cpc ) ;

Derivadas = [ dLdt
              dCAdt
              dCBdt
              dCCdt
              dTdt
              dTcdt ] ;
sys = Derivadas       ;

elseif abs(flag) == 3
  x  = x .* [ x > 0 ] ;

  L  = x(1) ; % nvel        do reator         [=] m
  Tc = x(6) ; % temperatura  da camisa         [=] C
  T  = x(5) ; % temperatura  do reator         [=] C
  
  % Transformando as variveis de estado em variveis de sada
  sys(1,1) = L  ; % nvel        do reator      [=] m
  sys(2,1) = Tc ; % temperatura  da camisa      [=] C
  sys(3,1) = T  ; % temperatura  do reator      [=] C

  % Fracao molar na descarga
  CM(A)    = x(2)         ; % concentracao molar de A [=] kmol/m
  CM(B)    = x(3)         ; % concentracao molar de B [=] kmol/m 
  CM(C)    = x(4)         ; % concentracao molar de C [=] kmol/m
  xM       = CM / sum(CM) ; % fraes molares
  sys(4,1) = xM(A)        ; % frao molar de A
  sys(5,1) = xM(B)        ; % frao molar de B
  sys(6,1) = xM(C)        ; % frao molar de C

  sys(7,1) = CM(A) ;
  sys(8,1) = CM(B) ;
  sys(9,1) = CM(C) ;

  % Autor:  Ricardo Kalid
%         kalid@ufba.br

elseif flag == 0
% Inicializao do sistema

CondInicCA = Estado_Estacionario(70) ; % [=] kmol/m
CondInicCB = Estado_Estacionario(71) ; % [=] kmol/m
CondInicCC = Estado_Estacionario(72) ; % [=] kmol/m
CondInicL  = Estado_Estacionario(73) ; % [=] m
CondInicT  = Estado_Estacionario(74) ; % [=] C
CondInicTc = Estado_Estacionario(75) ; % [=] C
        
CondicoesIniciais = [ CondInicL
                      CondInicCA
                      CondInicCB
                      CondInicCC
                      CondInicT
                      CondInicTc ] ;
                      
  x0       = CondicoesIniciais     ;
  sys(1,1) =  6 ; % Nmero de estados contnuos
  sys(2,1) =  0 ; % Nmero de estados discretos
  sys(3,1) =  9 ; % Nmero de sadas
  sys(4,1) =  9 ; % Nmero de entradas
  sys(5,1) =  0 ; % Nmero de raizes discretas
  sys(6,1) =  0 ; % Usado em sistemas de sistemas

else
  sys = [];  
end

% Autor:  Ricardo Kalid
%         kalid@ufba.br

% Fim deste arquivo