function [sys,x0] = TANQUE_sedo(t,x,u,flag,Estado_Estacionario)

% Planta: CSTR
%
% Autor:  Ricardo Kalid
%         kalid@ufba.br
%
% Interface no Simulink para  S_sedo.m     
%
% Inputs:
%
% t : tempo em minutos
% x : estados do sistema
% u : perturbaes externas ou variveis manipuladas
%
% Outputs:   sys  e  x0  como descritos no manual do SIMULINK
%            quando  flag    0 , sys contem as dimenses dos vetores e 
%            x0  contem as condies iniciais,
%            quando  flag    1 , sys contem as derivadas,
%            quando  flag    3 , sys contem as sadas: 

A = 1 ;
B = 2 ;
C = 3 ;

TipoVazaoDescarga = 2 ;

% Dados para a   PLANTA  do  TANQUE DE AQUECIMENTO
Vi    =   24.0       ; % [=] m
ro    =   19.20      ; % [=] kmol/m
rom   =    1.00      ; % [=] ton/m
Cp    =    1.815e5   ; % [=] J/(kmol.C)
Cp    =    0.2389*Cp ; % [=] kcal/(kmol.C)

Rg    = 8314.39    ; % [=] 
g     =    9.81    ; % [=] m/s

Rt    = ( 2 * Vi / ( 2 * pi ) ) ^ (1/3) ; % Lmax = 2R , e L = 50% Lmax
At    = pi * Rt ^ 2                     ; % rea transversal do tanque (L=2R) [=] m
Li    = Vi / At                         ; % nvel de projeto [=] m

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if abs(flag) == 1
% Clculo das derivadas dos estados
x   = x .* [ x > 0 ] ;
u   = u .* [ u > 0 ] ;

% Estados do sistema
L     = x(1) ; % nvel        do reator  [=] m
T     = x(2) ; % temperatura  do reator  [=] C
CM(A) = x(3) ; % concentracao molar de A [=] kmol/m
CM(B) = x(4) ; % concentracao molar de B [=] kmol/m
CM(C) = x(5) ; % concentracao molar de C [=] kmol/m

% Entradas do sistema
Fi    = u( 1) ; % vazo        da carga (make-up)       [=] ton/h
Ti    = u( 2) ; % temperatura  da carga (make-up)       [=] C
xi(A) = u( 3) ; % fracao molar do reagente A (make-up)  [=] (mol de A) / (mol mistura)
xi(B) = u( 4) ; % fracao molar do produto  B (make-up)  [=] (mol de B) / (mol mistura)
xi(C) = u( 5) ; % fracao molar do inerte   C (make-up)  [=] (mol de C) / (mol mistura)
F     = u( 6) ; % vazo        da descarga do tanque    [=] ton/h
Fr    = u( 7) ; % vazo        do reciclo               [=] ton/h
Tr    = u( 8) ; % temperatura  do reciclo               [=] C
xr(A) = u( 9) ; % frao molar do reagente A do reciclo [=] (mol de A) / (mol mistura)
xr(B) = u(10) ; % frao molar do produto  B do reciclo [=] (mol de B) / (mol mistura)
xr(C) = u(11) ; % frao molar do inerte   C do reciclo [=] (mol de C) / (mol mistura)
Qs    = u(12) ; % calor para serpentina                 [=] 

% Normalizacao das fracoes molares
xi = xi / sum( xi ) ;

if ( F <= eps | L <= eps | T <= eps )
   disp( '      F        L        T' )
   disp( [ F  L  T ] )
   disp( ' Problemas em TANQUE_sedo.M' )
   return
end

% Vazao molar
Fi_M = Fi * ro / rom ; % [=] kmol/h
Fr_M = Fr * ro / rom ; % [=] kmol/h
F_M  = F  * ro / rom ; % [=] kmol/h

% Fracao molar na descarga
xM = CM / sum(CM) ;

% Sistema de Equaes Diferenciais Ordinrias (SEDO)
V     = At * L                                                                                      ;
dLdt  = ( Fi + Fr - F ) / At                                                                        ;
dTdt  = ( Fi * ( Ti - T ) +  Fr * ( Tr - T ) ) / ( rom * V )     +  Qs / ( V * ro * Cp )            ;
dCAdt = ( ( xi(A) * Fi_M  +  xr(A) * Fr_M  - xM(A) * F_M )  -  CM(A) * ( Fi + Fr - F ) / rom ) /  V ;
dCBdt = ( ( xi(B) * Fi_M  +  xr(B) * Fr_M  - xM(B) * F_M )  -  CM(B) * ( Fi + Fr - F ) / rom ) /  V ;
dCCdt = ( ( xi(C) * Fi_M  +  xr(C) * Fr_M  - xM(C) * F_M )  -  CM(C) * ( Fi + Fr - F ) / rom ) /  V ;

Derivadas = [ dLdt
              dTdt
              dCAdt 
              dCBdt 
              dCCdt ] ;
sys = Derivadas       ;

elseif abs(flag) == 3
  x   = x .* [ x > 0 ] ;
  % Transformando as variveis de estado em variveis de sada
  
  % Vazo de descarga do reator
  L = x(1) ; % nvel                       [=] m
  T = x(2) ; % temperatura do reator       [=] C
  
  % Fracao molar na descarga
  CM(A)    = x(3)         ; % concentracao molar de A [=] kmol/m
  CM(B)    = x(4)         ; % concentracao molar de B [=] kmol/m 
  CM(C)    = x(5)         ; % concentracao molar de C [=] kmol/m
  xM       = CM / sum(CM) ; % fraes molares

  sys(1,1) = L     ; % nvel do tanque             [=] m
  sys(2,1) = T     ; % temperatura do reator       [=] C

  sys(3,1) = xM(A) ; % frao molar de A
  sys(4,1) = xM(B) ; % frao molar de B
  sys(5,1) = xM(C) ; % frao molar de C
  
  sys(6,1) = CM(A) ;
  sys(7,1) = CM(B) ;
  sys(8,1) = CM(C) ;
  
elseif flag == 0

% Inicializao do sistema
CondInicL  = Estado_Estacionario(1) ; % nivel do tanque de carga         [=] m
CondInicT  = Estado_Estacionario(2) ; % temperatura do tanque de carga   [=] C
CondInicA  = Estado_Estacionario(3) ; % concentracao molar do reagente A [=] kmol/m
CondInicB  = Estado_Estacionario(4) ; % concentracao molar do produto  B [=] kmol/m
CondInicC  = Estado_Estacionario(5) ; % concentracao molar do inerte   C [=] kmol/m
   
CondicoesIniciais = [ CondInicL
                      CondInicT
                      CondInicA
                      CondInicB
                      CondInicC ] ;
                      
  x0       = CondicoesIniciais    ;

  sys(1,1) =  5  ; % Nmero de estados contnuos
  sys(2,1) =  0  ; % Nmero de estados discretos
  sys(3,1) =  8  ; % Nmero de sadas
  sys(4,1) = 12  ; % Nmero de entradas
  sys(5,1) =  0  ; % Nmero de raizes discretas
  sys(6,1) =  0  ; % Usado em sistemas de sistemas

else
  sys = [];  

end

% Autor:  Ricardo Kalid
%         kalid@ufba.br

% Fim deste arquivo
