function [sys,x0] = TROCADOR_sedo(t,x,u,flag,Estado_Estacionario)

% Planta: TROCADOR
%
% Autor:  Ricardo Kalid
%         kalid@ufba.br
%
% Interface no Simulink para  TROCADOR_sedo.m     
%
% Inputs:
%
% t : tempo em segundos
% x : estados do sistema
% u : perturbaes externas ou variveis manipuladas
%
% Outputs:   sys  e  x0  como descritos no manual do SIMULINK
%            quando  flag    0 , sys contem as dimenses dos vetores e 
%            x0  contem as condies iniciais,
%            quando  flag    1 , sys contem as derivadas,
%            quando  flag    3 , sys contem as sadas: 

% Autor:  Ricardo Kalid
%         kalid@ufba.br

% Perda de calor para o meio ambiente pelo casco do trocador
% kk
Qperda = 0.5 ; % em relacao ao calor trocado do casco com o tubo

% Dados sobre o trocador de calor

Nelementos = 8        ;

CompTrocador = 18     ; % [=] m

CpT = 2 ; % calor especfico do fludo do tubo (kJ/kg*K)
CpS = 2 ; % calor especfico do fludo do casco (kJ/kg*K)
CpM = 1 ; % calor especfico do metal (kJ/kg*K)

L   = CompTrocador / ( Nelementos + 1 ) ; % comprimento de cada elemento

DT = 0.50 ; % dimetro interno do tubo (m)
DM = 0.51 ; % dimetro externo do tubo (m)
DS = 1.00 ; % dimetro interno do casco (m)

roT = 1500 ; % densidade fludo do tubo  (kg/m)
roM = 5000 ; % densidade do metal        (kg/m)
roS = 1000 ; % densidade fludo do casco (kg/m)

HTM = 20 ; % constante de troca trmica
HMS = 20 ; % constante de troca trmica


% reas e volumes dos elementos

AT  = pi * DT * L     ;
AM  = pi * DM * L     ;
VT  = pi * DT^2 * L/4 ;
VM  = pi * (DM^2 - DT^2) * L/4 ;
VS  = pi * (DS^2 - DM^2) * L/4 ;


% Fatores comuns nas equaes dos modelos

YT = CpT * VT * roT ;
YM = CpM * VM * roM ;
YS = CpS * VS * roS ;
UTM = HTM * AT ;
UMS = HMS * AM ;

% Autor:  Ricardo Kalid
%         kalid@ufba.br

% Variveis de entrada

TS9 =  20 + 273 ; % temperatura de entrada do fludo do casco (K)
TT0 = 102 + 273 ; % temperatura de entrada do fludo do tubo (K)

FT = 7.5e-3 ; % vazo de entrada do fludo do tubo  (m/s)
FS = 1.5e-3 ; % vazo de entrada do fludo do casco (m/s)

if abs(flag) == 1
  % Clculo das derivadas dos estados
  x   = x .* [ x > 0 ] ;
  u   = u .* [ u > 0 ] ;
  % Entradas do sistema

FT  = u(1)           ; % vazo       de entrada do fluido do tubo           [=] m/s
TT0 = u(2) + 273.15  ; % temperatira de entrada do fluido do tubo           [=] C
FS  = u(3)           ; % vazo       de entrada do fluido do casco          [=] m/s
TS9 = u(4) + 273.15  ; % temperatira de entrada do fluido do casco          [=] C


% Estados do sistema
TT = x(1:Nelementos)                 ; % temperatura do fluido no interior do tubo  [=] C
TS = x(Nelementos+1:2*Nelementos)    ; % temperatura do fluido no interior do casco [=] C
TM = x(2*Nelementos+1:3*Nelementos)  ; % temperatura da parede do tubo              [=] C

% Fatores comuns nas equaes dos modelos
ZT = FT / VT        ;
ZS = FS / VS        ;

% Sistema de Equaes Diferenciais Ordinrias (SEDO)
QT = UTM * ( TT - TM ) ;
QM = UMS * ( TM - TS ) ;

% Elemento 1
TS0    = TS(1) + 0.5 * ( TS(1) - TS(2) )                      ; % TS(saida) = TS(0)
% TS0    = TS(1)                                                ; % kk ->  TS(saida) = TS(1)
dTT1dt = ZT * ( TT0 - 0.5 * ( TT(1) + TT(2) ) ) - QT(1) / YT  ;
dTS1dt = ZS * ( 0.5 * ( TS(1) + TS(2) ) - TS0 ) + QM(1) / YS  ;
dTM1dt = ( QT(1) - QM(1) ) / YM                               ;

% Elementos 2 a 7
dTT27dt = ZT * ( TT(1:6) - TT(2:7) ) - QT(2:7) / YT           ;
dTS27dt = ZS * ( TS(3:8) - TS(2:7) ) + QM(2:7) / YS           ;
dTM27dt = ( QT(2:7) - QM(2:7) ) / YM                          ;

% Elemento 8
TT9    = TT(8) - 0.5 * ( TT(7) - TT(8) )                      ;
dTT8dt = ZT * ( 0.5 * ( TT(7) + TT(8) ) - TT9 ) - QT(8) / YT  ;
dTS8dt = ZS * ( TS9 - 0.5 * ( TS(7) + TS(8) ) ) + QM(8) / YS  ;
dTM8dt = ( QT(8) - QM(8) ) / YM                               ;

Derivadas = [ dTT1dt
              dTT27dt
              dTT8dt
              dTS1dt
              dTS27dt
              dTS8dt
              dTM1dt
              dTM27dt
              dTM8dt   ]  ;

sys = Derivadas ;

elseif abs(flag) == 3
  x   = x .* [ x > 0 ] ;
  u   = u .* [ u > 0 ] ;
  % Transformando as variveis de estado em variveis de sada

  TT(1:8) = x(1:Nelementos)                 ; % temperatura do fluido no interior do tubo  [=] K
  TT(9)   = TT(8) - 0.5 * ( TT(7) - TT(8) ) ; % temperatura do fluido na sada    do tubo  [=] K
  sys(1:Nelementos+1,1) = TT' - 273.15      ; % temperatura ao longo do tubo               [=] C

  TS(2:9) = x(Nelementos+1:2*Nelementos)                     ; % temperatura do fluido no interior do casco  [=] K
  TS(1)   = TS(2)                                            ; % kk ->  TS(saida) = TS(1)
%  TS(1)   = TS(2) + 0.5 * ( TS(2) - TS(3) )                  ; % temperatura do fluido na sada    do casco  [=] K
  sys((Nelementos+1)+1:2*(Nelementos+1)) = TS' - 273.15      ; % temperatura ao longo do casco               [=] C

  TM(1:8) = x(2*Nelementos+1:3*Nelementos)                   ; % temperatura da parede do tubo              [=] K
  sys(2*(Nelementos+1)+1:3*(Nelementos+1)-1) = TM' - 273.15  ; % temperatura ao longo da parede do tubo     [=] C

% Autor:  Ricardo Kalid
%         kalid@ufba.br

elseif flag == 0
% Inicializao do sistema

Nelementos   = 8      ;
CompTrocador = 18     ; % [=] m

TTinicial = [ Estado_Estacionario(14:21)' ] + 273.15 ;

TSinicial = [ Estado_Estacionario(22:29)' ] + 273.15 ;

TMinicial = [ Estado_Estacionario(30:37)' ] + 273.15 ;

Trocador_NumEstados  = 3 * Nelementos ;
Trocador_NumEntradas = 4              ;

Trocador_CondicoesIniciais = [ TTinicial
                               TSinicial
                               TMinicial ] ;
                      
  x0       = Trocador_CondicoesIniciais ;
  sys(1,1) = Trocador_NumEstados        ; % Nmero de estados contnuos
  sys(2,1) = 0                          ; % Nmero de estados discretos
  sys(3,1) = Trocador_NumEstados + 2    ; % Nmero de sadas
  sys(4,1) = Trocador_NumEntradas       ; % Nmero de entradas
  sys(5,1) = 0                          ; % Nmero de raizes discretas
  sys(6,1) = 0                          ; % Usado em sistemas de sistemas

else
  sys = [];  

end

% Autor:  Ricardo Kalid
%         kalid@ufba.br

% Fim deste arquivo
